#include <cstdlib>
#include <cmath>
#include <iostream>
#include <cstdio>
#include <string>
#include <fstream>
#include <vector>

#include <glwidget.h>
#include <glm/gtc/type_precision.hpp>

GLwidget::GLwidget()
{
	// Initialise GLFW
	if( !glfwInit() )
	{
		std::cout << "Failed to initialize GLFW" << std::endl;
		exit(EXIT_FAILURE);
	}

	glfwOpenWindowHint(GLFW_FSAA_SAMPLES, 4); //Anti Aliasing
	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MAJOR, 2); //OpenGL 2.1
	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MINOR, 1);

	// Open 1024x768 a window and create its OpenGL context
	width = 1024;
	height = 768;
	if( !glfwOpenWindow( width, height, 0,0,0,0, 32,0, GLFW_WINDOW ) )
	{
		std::cout << "Failed to open GLFW window" << std::endl;
		glfwTerminate();
		exit(EXIT_FAILURE);
	}
	// Set a title
	glfwSetWindowTitle( "Polytech RICM 4" );
	// Enable GLFW to receive key press
	glfwEnable( GLFW_STICKY_KEYS );

	// Initialize GLEW
	if (glewInit() != GLEW_OK) {
		std::cout << "Failed to initialize GLEW" << std::endl;
		exit(EXIT_FAILURE);
	}

	//Get OpenGL version informations
	const GLubyte* renderer = glGetString (GL_RENDERER); // get renderer string
	const GLubyte* version = glGetString (GL_VERSION); // version as a string
	std::cout << "Renderer : " << renderer << std::endl;
	std::cout << "OpenGL version supported : " << version << std::endl;

	//Build data
	vec3 p0(0.0f, 0.5f, 0.0f);
	vec3 p1(0.5f, -0.5f, 0.0f);
	vec3 p2(-0.5f, -0.5f, 0.0f);
	vertices.resize(3);
	vertices.push_back(p0);
	vertices.push_back(p1);
	vertices.push_back(p2);
}

GLwidget::~GLwidget()
{
	glfwTerminate();
}

void GLwidget::init(const char* filename)
{
	// Dark blue background
	glClearColor(0.5f, 0.5f, 0.5f, 1.0f);

	// Create and compile our GLSL program from the shaders
	programID = LoadShaders( "vertex.glsl", "fragment.glsl" );

	// Use our shader
	glUseProgram(programID);

	// Get an id for our buffers
	vertexPositionID = glGetAttribLocation(programID, "vertex_position");

	/*Copy vertex onto the graphics cards in a vertex buffer object*/

	// Generate an empty buffer whose id is vertexBufferObject
	glGenBuffers(1, &vertexBufferObject);
	// Set the buffer as the current buffer in OpenGL state machine
	glBindBuffer(GL_ARRAY_BUFFER, vertexBufferObject);
	// Copy the points into the current buffer
	glBufferData(GL_ARRAY_BUFFER, vertices.size()*sizeof(glm::vec3), vertices.data(), GL_STATIC_DRAW);

	//Stop using our shader
	glUseProgram(0);
}



void GLwidget::displayLoop()
{
	do{

		// Clear the screen
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

		// Use our shader
		glUseProgram(programID);

		// Draw our buffer via shader pipeline
		renderShader();

		// Terminate using our shader
		glUseProgram(0);

		// Draw some stuff via fix pipeline
		renderFix();

		// Intercept mouse and keyboard
		eventHandler();

		// Swap buffers
		glfwSwapBuffers();

	} // Check if the ESC key was pressed or the window was closed
	while( glfwGetKey( GLFW_KEY_ESC ) != GLFW_PRESS &&
			glfwGetWindowParam( GLFW_OPENED ) );

	// Close OpenGL window and terminate GLFW
	glfwTerminate();

	// Cleanup VBO and shader program
	glDeleteBuffers(1, &vertexBufferObject);
	glDeleteProgram(programID);
}

void GLwidget::eventHandler()
{
    if (glfwGetKey('Q') == GLFW_PRESS)
    {
        camera.rotateY(1.0);
    }
    else if (glfwGetKey('D') == GLFW_PRESS)
    {
        camera.rotateY(-1.0);
    }
    else if (glfwGetKey('Z') == GLFW_PRESS)
    {
        camera.rotateX(1.0);
    }
    else if (glfwGetKey('S') == GLFW_PRESS)
    {
        camera.rotateX(-1.0);
    }
}

void GLwidget::renderShader()
{
	//Enable a vertex attribute with the id we chose before
	glEnableVertexAttribArray(vertexPositionID);
	//Define a layout for the attribute vertexPositionID and bind it to vertexBufferObject
	glBindBuffer(GL_ARRAY_BUFFER, vertexBufferObject);
	glVertexAttribPointer(
			vertexPositionID, // The attribute we want to configure
			3,					//size = 3 components per vertex
			GL_FLOAT,           // type
			GL_FALSE,           // normalized?
			0,                  // stride
			(void*)0            // element array buffer offset
			);

	// Draw the triangle using index buffer
	glDrawArrays(GL_TRIANGLES, 0, vertices.size());// 3 indices starting at 0 -> 1 triangle
	glDisableVertexAttribArray(vertexPositionID);
}

void GLwidget::renderFix(){}
